==> user_referral-3.0.2.tar.gz: user_referral/src/Entity/UserReferralType.php <==
<?php

namespace Drupal\user_referral\Entity;

use Drupal\Core\Config\Entity\ConfigEntityBase;
use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\user_referral\UserReferralTypeInterface;
use Drupal\user\UserInterface;
use Drupal\user\Entity\User;
use Drupal\Core\Url;
use Symfony\Component\HttpFoundation\Request;
use Drupal\Core\Language\LanguageInterface;
use Drupal\user_referral\Event\UserReferralCookieEvent;
use Drupal\user_referral\Event\UserReferralReferralEntryEvent;
use Drupal\user_referral\UserReferral;

/**
 * Defines the User Referral type configuration entity.
 *
 * @ConfigEntityType(
 *   id = "user_referral_type",
 *   label = @Translation("User referral type"),
 *   label_collection = @Translation("User referral types"),
 *   label_singular = @Translation("user referral type"),
 *   label_plural = @Translation("user referral types"),
 *   label_count = @PluralTranslation(
 *     singular = "@count user referral type",
 *     plural = "@count user referral types",
 *   ),
 *   handlers = {
 *     "access" = "Drupal\user_referral\UserReferralTypeAccessControlHandler",
 *     "form" = {
 *       "add" = "Drupal\user_referral\UserReferralTypeForm",
 *       "edit" = "Drupal\user_referral\UserReferralTypeForm",
 *       "delete" = "Drupal\user_referral\Form\UserReferralTypeDeleteConfirm"
 *     },
 *     "list_builder" = "Drupal\user_referral\UserReferralTypeListBuilder",
 *   },
 *   admin_permission = "administer user referral types",
 *   config_prefix = "type",
 *   entity_keys = {
 *     "id" = "type",
 *     "weight" = "weight",
 *     "label" = "name"
 *   },
 *   links = {
 *     "canonical" = "/admin/structure/user-referral-types/manage/{user_referral_type}",
 *     "edit-form" = "/admin/structure/user-referral-types/manage/{user_referral_type}",
 *     "delete-form" = "/admin/structure/user-referral-types/manage/{user_referral}/delete",
 *     "collection" = "/admin/structure/user-referral-types",
 *   },
 *   config_export = {
 *     "name",
 *     "type",
 *     "referral_field",
 *     "description",
 *     "url_alias_pattern",
 *     "cookie_lifetime",
 *     "goto_path",
 *     "roles",
 *     "reset_cookie_lifetime_for_same_referrer",
 *     "change_referrer_in_cookie",
 *     "weight",
 *   }
 * )
 */
class UserReferralType extends ConfigEntityBase implements UserReferralTypeInterface {

  /**
   * The machine name of this user referral type.
   *
   * @var string
   *
   * @todo Rename to $id.
   */
  protected $type;

  /**
   * The human-readable name of the referral type.
   *
   * @var string
   *
   * @todo Rename to $label.
   */
  protected $name;

  /**
   * A brief description of this referral type.
   *
   * @var string
   */
  protected $description;

  /**
   * The referral ID field to be used for this referral type.
   *
   * @var string
   */
  protected $referral_field;

  /**
   * A url_alias_pattern of this referral type.
   *
   * @var string
   */
  protected $url_alias_pattern;

  /**
   * Cookie lifetime in days.
   *
   * @var string
   */
  protected $cookie_lifetime = 7;

  /**
   * A path to which user to be redirect to after accesss a referral link.
   *
   * @var string
   */
  protected $goto_path = '/user/register';

  /**
   * A path to which user to be redirect to after accesss a referral link.
   *
   * @var array
   */
  protected $roles = [];

  /**
   * Whether to extend lifetime of cookie if user access referral link from same user.
   */
  protected $reset_cookie_lifetime_for_same_referrer = TRUE;

  /**
   * Whether to extend lifetime of cookie if user access referral link from same user.
   */
  protected $change_referrer_in_cookie = TRUE;

  /**
   * {@inheritdoc}
   */
  public function id() {
    return $this->type;
  }

  public function label() {
    return parent::label();
  }

  /**
   * {@inheritdoc}
   */
  public function isLocked() {
    $locked = \Drupal::state()->get('user_referral.type.locked');
    return isset($locked[$this->id()]) ? $locked[$this->id()] : FALSE;
  }


  /**
   * {@inheritdoc}
   */
  public function getDescription() {
    return $this->description;
  }

  /**
   * {@inheritdoc}
   */
  public function getReferralField() {
    return $this->referral_field;
  }

  /**
   * {@inheritdoc}
   */
  public function getURLAliasPattern() {
    return $this->url_alias_pattern;
  }

  /**
   * Return current URL alias path of the referral link for given user account.
   */
  public function getURLAlias(UserInterface $account) {
    $token_data = ['user' => $account];
    $token_data['user_referral_type'] = $this;
    $alias = \Drupal::token()->replace($this->url_alias_pattern, $token_data);
    return str_replace('_', '-', $alias);
  }

  /**
   * {@inheritdoc}
   */
  public function getCookieLifetimeInDays() {
    return $this->cookie_lifetime;
  }

  /**
   * {@inheritdoc}
   */
  public function getCookieLifetimeInSeconds() {
    return $this->cookie_lifetime * 86400;
  }

  /**
   * {@inheritdoc}
   */
  public function getGotoPath() {
    return $this->goto_path;
  }

  /**
   * {@inheritdoc}
   */
  public function getRoles() {
    // Return plain array to make it compatible with User::getRoles().
    return array_filter(array_values($this->roles));
  }

  /**
   * {@inheritdoc}
   */
  public function getWeight() {
    return $this->weight;
  }

  public function allowedToExtendCookieLifetimeForSameReferrer() {
    return $this->reset_cookie_lifetime_for_same_referrer;
  }

  public function allowedToChangeReferrer() {
    return $this->change_referrer_in_cookie;
  }

  /**
   * {@inheritdoc}
   */
  public static function postDelete(EntityStorageInterface $storage, array $entities) {
    parent::postDelete($storage, $entities);

    // Clear the node type cache to reflect the removal.
    $storage->resetCache(array_keys($entities));
  }

  /**
   * {@inheritdoc}
   */
  function setCookie(UserInterface $referrer, Request $request) {
    $cookie = new \stdClass();
    $cookie->uid = $referrer->id();
    $cookie->timestamp = time();
    $cookie->ip = $request->getClientIp();
    $cookie->referer  = $request->getUri();
    $cookie->type = $this->id();

    $event = new UserReferralCookieEvent($cookie);
    \Drupal::service('event_dispatcher')->dispatch($event, UserReferralCookieEvent::COOKIE_PRE_ASSIGN_LOGIC);

    $existing_cookie = isset($_COOKIE[self::COOKIE_NAME]) ? json_decode($_COOKIE[self::COOKIE_NAME]) : NULL;

    if ($existing_cookie) {
      if ($existing_cookie->type == $cookie->type) {
        if ($existing_cookie->uid == $cookie->uid) {
          if (!$this->allowedToExtendCookieLifetimeForSameReferrer()) {
            // Set not to extend cookie lifetime.
            // So continue with existing cookie.
            return;
          }
        }
        else if ($existing_cookie->uid != $cookie->uid && !$this->allowedToChangeReferrer()) {
          // User visiting referral link from different user,
          // but changing referrer is not allowed.
          return;
        }
      }
      else if (!\Drupal::configFactory()->get('user_referral.settings')->get('allow_to_change_referral_type')) {
        // Changing user referral type is not allowed.
        return;
      }
    }
    $event = new UserReferralCookieEvent($cookie);

    \Drupal::service('event_dispatcher')->dispatch($event, UserReferralCookieEvent::COOKIE_SET);
    

    setcookie(self::COOKIE_NAME, json_encode($event->getCookie()), time() + $this->getCookieLifetimeInSeconds(), '/');
  }

  /**
   * Get referral ID for an account.
   *
   * @return string
   *  Referral ID.
   */
  public function getAccountReferralID(UserInterface $account) {
    $referral_id = NULL;
    $referral_field_item = $account->get($this->getReferralField())->first();
    if (!empty($referral_field_item)) {
      $referral_id = $referral_field_item->getValue()['value'];
    }
    return $referral_id;
  }

  /**
   * Find user account for referral ID.
   *
   * @param string
   *  Referral ID
   * @return \Drupal\user\UserInterface
   *  User account
   */
  public function getReferralIDAccount($referral_id) {
    if ($referral_field = $this->getReferralField()) {
      // Note that we ignore node access.
      $uids = \Drupal::entityQuery('user')
        ->condition($this->getReferralField(), $referral_id)
        ->accessCheck(FALSE)
        ->execute();
      if (!empty($uids)) {
        $uid = reset($uids);
        return User::load($uid);
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function getReferralLink(UserInterface $account, $options = []) {
    if (empty($options['base_path'])) {
      $options['base_path'] = \Drupal::request()->getSchemeAndHttpHost() . \Drupal::request()->getBasePath();
    }
    $lang = LanguageInterface::LANGCODE_NOT_SPECIFIED;

    $url = Url::fromRoute('user_referral.referral_link', ['uid' => $account->id(), 'type' => str_replace('_', '-', $this->id())]);
    $internal_path = '/' . $url->getInternalPath();
    // Initilize final path same as internal.
    $final_path = $internal_path;

    $referral_id = $this->getAccountReferralID($account);
    $configured_alias = NULL;

    if ($referral_id) {
      // Alias will not be generated without Referral ID.
      $configured_alias = $this->getURLAlias($account);
    }

    $alias_manager = \Drupal::service('path_alias.manager');


    if (!empty($configured_alias) && !empty($referral_id)) {
      // Returned path will be same as alias if alias does not exist.
      $path = $alias_manager->getPathByAlias($configured_alias, $lang);
      if ($path != $configured_alias) {
        // Alias exists.
        if ($path != $internal_path) {
          // Alias exists but pointing to different path.
          // Generate alias with number suffix.
          $suffix_num = 0;
          $path_alias = NULL;
          while ($path_alias != NULL) {
            $configured_alias_with_suffix = $configured_alias . '-' . $suffix_num;
            $path_for_alias_with_suffix = $alias_manager->getPathByAlias($configured_alias_with_suffix, $lang);
            if ($path_for_alias_with_suffix == $configured_alias_with_suffix) {
              // This alias does not exist.
              $path_alias = \Drupal::entityTypeManager()->getStorage('path_alias')->create([
                'path' => $internal_path,
                'alias' => $path_for_alias_with_suffix,
                'langcode' => $lang,
              ]);
              $path_alias->save();
              $final_path = $configured_alias;
              break;
            }
            else if ($path_for_alias_with_suffix == $internal_path) {
              // This alias exists and ponting correctly.
              $final_path = $configured_alias_with_suffix;
              break;
            }
            $suffix_num++;
          }
        }
        else {
          $final_path = $configured_alias;
        }
      }
      else {
        $path_alias = \Drupal::entityTypeManager()->getStorage('path_alias')->create([
          'path' => $internal_path,
          'alias' => $configured_alias,
          'langcode' => $lang,
        ]);
        $path_alias->save();
        $final_path = $configured_alias;
      }
    }

    return Url::fromUri($options['base_path'] . $final_path);
  }

  /**
   * Decides whether referral links of this type is applicable for give user account.
   */
  public function isApplicable($account) {
    $roles = $this->getRoles();
    $user_roles = $account->getRoles();
    return !empty(array_intersect($roles, $user_roles));
  }

  /**
   * Act on new user account creation.
   * This method checks if referral cookies exist and it will create referral entry as needed.
   */
  public static function processUserCreation(UserInterface $account) {
    if (!isset($_COOKIE[self::COOKIE_NAME])) {
      return;
    }

    // Retrieve referral info from the cookie
    $cookie = json_decode($_COOKIE[self::COOKIE_NAME]);

    if (empty($cookie)) {
      // Nothing to do ...
      return;
    }

    $entry = array(
      'uid' => $account->id(),
      'referrer_uid' => $cookie->uid,
      'created' => $cookie->timestamp,
      'host' => $cookie->ip,
      'http_referer' => $cookie->referer,
      'type' => $cookie->type,
    );

    $event = new UserReferralReferralEntryEvent($entry, $cookie);

    \Drupal::service('event_dispatcher')->dispatch($event, UserReferralReferralEntryEvent::ENTRY_CREATE);
    UserReferral::createReferralEntry($event->getEntry());
  }

  public static function referralTypeOptions() {
    $user_referral_types = self::loadMultiple();
    $options = [];
    foreach ($user_referral_types as $referral_type) {
      $options[$referral_type->id()] = $referral_type->label();
    }
    return $options;
  }

  /**
   * {@inheritdoc}
   */
  public static function postLoad(EntityStorageInterface $storage, array &$entities) {
    parent::postLoad($storage, $entities);
    // Sort the queried referral types by their weight.
    // See \Drupal\Core\Config\Entity\ConfigEntityBase::sort().
    uasort($entities,  static::sort(...));
  }

}

==> user_referral-3.1.1.tar.gz: user_referral/src/Entity/UserReferralType.php <==
<?php

namespace Drupal\user_referral\Entity;

use Drupal\Core\Config\Entity\ConfigEntityBase;
use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\user_referral\UserReferralTypeInterface;
use Drupal\user\UserInterface;
use Drupal\user\Entity\User;
use Drupal\Core\Url;
use Symfony\Component\HttpFoundation\Request;
use Drupal\Core\Language\LanguageInterface;
use Drupal\user_referral\Event\UserReferralCookieEvent;
use Drupal\user_referral\Event\UserReferralReferralEntryEvent;
use Drupal\user_referral\UserReferral;

/**
 * Defines the User Referral type configuration entity.
 *
 * @ConfigEntityType(
 *   id = "user_referral_type",
 *   label = @Translation("User referral type"),
 *   label_collection = @Translation("User referral types"),
 *   label_singular = @Translation("user referral type"),
 *   label_plural = @Translation("user referral types"),
 *   label_count = @PluralTranslation(
 *     singular = "@count user referral type",
 *     plural = "@count user referral types",
 *   ),
 *   handlers = {
 *     "access" = "Drupal\user_referral\UserReferralTypeAccessControlHandler",
 *     "form" = {
 *       "add" = "Drupal\user_referral\UserReferralTypeForm",
 *       "edit" = "Drupal\user_referral\UserReferralTypeForm",
 *       "delete" = "Drupal\user_referral\Form\UserReferralTypeDeleteConfirm"
 *     },
 *     "list_builder" = "Drupal\user_referral\UserReferralTypeListBuilder",
 *   },
 *   admin_permission = "administer user referral types",
 *   config_prefix = "type",
 *   entity_keys = {
 *     "id" = "type",
 *     "weight" = "weight",
 *     "label" = "name"
 *   },
 *   links = {
 *     "canonical" = "/admin/structure/user-referral-types/manage/{user_referral_type}",
 *     "edit-form" = "/admin/structure/user-referral-types/manage/{user_referral_type}",
 *     "delete-form" = "/admin/structure/user-referral-types/manage/{user_referral}/delete",
 *     "collection" = "/admin/structure/user-referral-types",
 *   },
 *   config_export = {
 *     "name",
 *     "type",
 *     "referral_field",
 *     "description",
 *     "url_alias_pattern",
 *     "cookie_lifetime",
 *     "goto_path",
 *     "roles",
 *     "reset_cookie_lifetime_for_same_referrer",
 *     "change_referrer_in_cookie",
 *     "weight",
 *   }
 * )
 */
class UserReferralType extends ConfigEntityBase implements UserReferralTypeInterface {

  /**
   * The machine name of this user referral type.
   *
   * @var string
   *
   * @todo Rename to $id.
   */
  protected $type;

  /**
   * The human-readable name of the referral type.
   *
   * @var string
   *
   * @todo Rename to $label.
   */
  protected $name;

  /**
   * A brief description of this referral type.
   *
   * @var string
   */
  protected $description;

  /**
   * The referral ID field to be used for this referral type.
   *
   * @var string
   */
  protected $referral_field;

  /**
   * A url_alias_pattern of this referral type.
   *
   * @var string
   */
  protected $url_alias_pattern;

  /**
   * Cookie lifetime in days.
   *
   * @var string
   */
  protected $cookie_lifetime = 7;

  /**
   * A path to which user to be redirect to after accesss a referral link.
   *
   * @var string
   */
  protected $goto_path = '/user/register';

  /**
   * A path to which user to be redirect to after accesss a referral link.
   *
   * @var array
   */
  protected $roles = [];

  /**
   * Whether to extend lifetime of cookie if user access referral link from same user.
   */
  protected $reset_cookie_lifetime_for_same_referrer = TRUE;

  /**
   * Whether to extend lifetime of cookie if user access referral link from same user.
   */
  protected $change_referrer_in_cookie = TRUE;

  /**
   * {@inheritdoc}
   */
  public function id() {
    return $this->type;
  }

  public function label() {
    return parent::label();
  }

  /**
   * {@inheritdoc}
   */
  public function isLocked() {
    $locked = \Drupal::state()->get('user_referral.type.locked');
    return isset($locked[$this->id()]) ? $locked[$this->id()] : FALSE;
  }


  /**
   * {@inheritdoc}
   */
  public function getDescription() {
    return $this->description;
  }

  /**
   * {@inheritdoc}
   */
  public function getReferralField() {
    return $this->referral_field;
  }

  /**
   * {@inheritdoc}
   */
  public function getURLAliasPattern() {
    return $this->url_alias_pattern;
  }

  /**
   * Return current URL alias path of the referral link for given user account.
   */
  public function getURLAlias(UserInterface $account) {
    $token_data = ['user' => $account];
    $token_data['user_referral_type'] = $this;
    $alias = \Drupal::token()->replace($this->url_alias_pattern, $token_data);
    return str_replace('_', '-', $alias);
  }

  /**
   * {@inheritdoc}
   */
  public function getCookieLifetimeInDays() {
    return $this->cookie_lifetime;
  }

  /**
   * {@inheritdoc}
   */
  public function getCookieLifetimeInSeconds() {
    return $this->cookie_lifetime * 86400;
  }

  /**
   * {@inheritdoc}
   */
  public function getGotoPath() {
    return $this->goto_path;
  }

  /**
   * {@inheritdoc}
   */
  public function getRoles() {
    // Return plain array to make it compatible with User::getRoles().
    return array_filter(array_values($this->roles));
  }

  /**
   * {@inheritdoc}
   */
  public function getWeight() {
    return $this->weight;
  }

  public function allowedToExtendCookieLifetimeForSameReferrer() {
    return $this->reset_cookie_lifetime_for_same_referrer;
  }

  public function allowedToChangeReferrer() {
    return $this->change_referrer_in_cookie;
  }

  /**
   * {@inheritdoc}
   */
  public static function postDelete(EntityStorageInterface $storage, array $entities) {
    parent::postDelete($storage, $entities);

    // Clear the node type cache to reflect the removal.
    $storage->resetCache(array_keys($entities));
  }

  /**
   * {@inheritdoc}
   */
  function setCookie(UserInterface $referrer, Request $request) {
    $cookie = new \stdClass();
    $cookie->uid = $referrer->id();
    $cookie->timestamp = time();
    $cookie->ip = $request->getClientIp();
    $cookie->referer  = $request->getUri();
    $cookie->type = $this->id();

    $event = new UserReferralCookieEvent($cookie);
    \Drupal::service('event_dispatcher')->dispatch($event, UserReferralCookieEvent::COOKIE_PRE_ASSIGN_LOGIC);

    $request = \Drupal::request();

    $cookie_data = $request->cookies->get(UserReferralType::COOKIE_NAME);
    $existing_cookie = isset($cookie_data) ? json_decode($cookie_data) : NULL;

    if ($existing_cookie) {
      if ($existing_cookie->type == $cookie->type) {
        if ($existing_cookie->uid == $cookie->uid) {
          if (!$this->allowedToExtendCookieLifetimeForSameReferrer()) {
            // Set not to extend cookie lifetime.
            // So continue with existing cookie.
            return;
          }
        }
        else if ($existing_cookie->uid != $cookie->uid && !$this->allowedToChangeReferrer()) {
          // User visiting referral link from different user,
          // but changing referrer is not allowed.
          return;
        }
      }
      else if (!\Drupal::configFactory()->get('user_referral.settings')->get('allow_to_change_referral_type')) {
        // Changing user referral type is not allowed.
        return;
      }
    }
    $event = new UserReferralCookieEvent($cookie);

    \Drupal::service('event_dispatcher')->dispatch($event, UserReferralCookieEvent::COOKIE_SET);
    
    $cookie_raw = json_encode($event->getCookie());
    setcookie(self::COOKIE_NAME, $cookie_raw, time() + $this->getCookieLifetimeInSeconds(), '/');
    $request->cookies->set(self::COOKIE_NAME, $cookie_raw);
  }

  /**
   * Get referral ID for an account.
   *
   * @return string
   *  Referral ID.
   */
  public function getAccountReferralID(UserInterface $account) {
    $referral_id = NULL;
    $referral_field_item = $account->get($this->getReferralField())->first();
    if (!empty($referral_field_item)) {
      $referral_id = $referral_field_item->getValue()['value'];
    }
    return $referral_id;
  }

  /**
   * Find user account for referral ID.
   *
   * @param string
   *  Referral ID
   * @return \Drupal\user\UserInterface
   *  User account
   */
  public function getReferralIDAccount($referral_id) {
    if ($referral_field = $this->getReferralField()) {
      // Note that we ignore node access.
      $uids = \Drupal::entityQuery('user')
        ->condition($this->getReferralField(), $referral_id)
        ->accessCheck(FALSE)
        ->execute();
      if (!empty($uids)) {
        $uid = reset($uids);
        return User::load($uid);
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function getReferralLink(UserInterface $account, $options = []) {
    if (empty($options['base_path'])) {
      $options['base_path'] = \Drupal::request()->getSchemeAndHttpHost() . \Drupal::request()->getBasePath();
    }
    $lang = LanguageInterface::LANGCODE_NOT_SPECIFIED;

    $url = Url::fromRoute('user_referral.referral_link', ['uid' => $account->id(), 'type' => str_replace('_', '-', $this->id())]);
    $internal_path = '/' . $url->getInternalPath();
    // Initilize final path same as internal.
    $final_path = $internal_path;

    $referral_id = $this->getAccountReferralID($account);
    $configured_alias = NULL;

    if ($referral_id) {
      // Alias will not be generated without Referral ID.
      $configured_alias = $this->getURLAlias($account);
    }

    $alias_manager = \Drupal::service('path_alias.manager');


    if (!empty($configured_alias) && !empty($referral_id)) {
      // Returned path will be same as alias if alias does not exist.
      $path = $alias_manager->getPathByAlias($configured_alias, $lang);
      if ($path != $configured_alias) {
        // Alias exists.
        if ($path != $internal_path) {
          // Alias exists but pointing to different path.
          // Generate alias with number suffix.
          $suffix_num = 0;
          $path_alias = NULL;
          while ($path_alias != NULL) {
            $configured_alias_with_suffix = $configured_alias . '-' . $suffix_num;
            $path_for_alias_with_suffix = $alias_manager->getPathByAlias($configured_alias_with_suffix, $lang);
            if ($path_for_alias_with_suffix == $configured_alias_with_suffix) {
              // This alias does not exist.
              $path_alias = \Drupal::entityTypeManager()->getStorage('path_alias')->create([
                'path' => $internal_path,
                'alias' => $path_for_alias_with_suffix,
                'langcode' => $lang,
              ]);
              $path_alias->save();
              $final_path = $configured_alias;
              break;
            }
            else if ($path_for_alias_with_suffix == $internal_path) {
              // This alias exists and ponting correctly.
              $final_path = $configured_alias_with_suffix;
              break;
            }
            $suffix_num++;
          }
        }
        else {
          $final_path = $configured_alias;
        }
      }
      else {
        $path_alias = \Drupal::entityTypeManager()->getStorage('path_alias')->create([
          'path' => $internal_path,
          'alias' => $configured_alias,
          'langcode' => $lang,
        ]);
        $path_alias->save();
        $final_path = $configured_alias;
      }
    }

    return Url::fromUri($options['base_path'] . $final_path);
  }

  /**
   * Decides whether referral links of this type is applicable for give user account.
   */
  public function isApplicable($account) {
    $roles = $this->getRoles();
    $user_roles = $account->getRoles();
    return !empty(array_intersect($roles, $user_roles));
  }

  /**
   * Act on new user account creation.
   * This method checks if referral cookies exist and it will create referral entry as needed.
   */
  public static function processUserCreation(UserInterface $account) {
    $request = \Drupal::request();

    $cookie_data = $request->cookies->get(UserReferralType::COOKIE_NAME);
    if (!isset($cookie_data)) {
      return;
    }

    // Retrieve referral info from the cookie
    $cookie = json_decode($cookie_data);

    if (empty($cookie)) {
      // Nothing to do ...
      return;
    }

    $entry = array(
      'uid' => $account->id(),
      'referrer_uid' => $cookie->uid,
      'created' => $cookie->timestamp,
      'host' => $cookie->ip,
      'http_referer' => $cookie->referer,
      'type' => $cookie->type,
    );

    $event = new UserReferralReferralEntryEvent($entry, $cookie);

    \Drupal::service('event_dispatcher')->dispatch($event, UserReferralReferralEntryEvent::ENTRY_CREATE);
    UserReferral::createReferralEntry($event->getEntry());
  }

  public static function referralTypeOptions() {
    $user_referral_types = self::loadMultiple();
    $options = [];
    foreach ($user_referral_types as $referral_type) {
      $options[$referral_type->id()] = $referral_type->label();
    }
    return $options;
  }

  /**
   * {@inheritdoc}
   */
  public static function postLoad(EntityStorageInterface $storage, array &$entities) {
    parent::postLoad($storage, $entities);
    // Sort the queried referral types by their weight.
    // See \Drupal\Core\Config\Entity\ConfigEntityBase::sort().
    uasort($entities,  static::sort(...));
  }

}
